var _this = this;
import * as tslib_1 from "tslib";
import { drop, every, first, has, isNil, map, reverse, } from "lodash";
import { FetchFakeMiddleware } from "./middlewares/fetch";
import { getRetryMiddlewareDeclaration, } from "./middlewares/retry";
export var validateMiddlewareDeclarations = function (middlewares) {
    return every(map(middlewares, function (middlewareDeclaration) {
        return has(middlewareDeclaration, "config")
            && !isNil(middlewareDeclaration.class);
    }));
};
var validateFetchyConfig = function (fetchyConfig) {
    if (isNil(fetchyConfig.middlewares)) {
        return [];
    }
    return validateMiddlewareDeclarations(fetchyConfig.middlewares)
        ? fetchyConfig.middlewares
        : null;
};
var instanceFetchyMiddleware = function (middlewareClass, middlewareConfig, nextRing) {
    if (nextRing === null) {
        nextRing = new FetchFakeMiddleware({}, null);
    }
    return new middlewareClass(middlewareConfig, nextRing);
};
var buildChainRings = function (middlewareDeclaration, nextRing, remainingMiddlewareDeclarations) {
    var ring = instanceFetchyMiddleware(middlewareDeclaration.class, middlewareDeclaration.config, nextRing);
    if (remainingMiddlewareDeclarations.length > 0) {
        return buildChainRings(first(remainingMiddlewareDeclarations), ring, drop(remainingMiddlewareDeclarations, 1));
    }
    return ring;
};
export var buildChain = function (fetchyConfig) {
    var middlewareDeclarations = validateFetchyConfig(fetchyConfig);
    if (middlewareDeclarations === null) {
        throw new Error("middlewares object is not valid");
    }
    var retryMiddlewareDeclaration = getRetryMiddlewareDeclaration(fetchyConfig.retry);
    if (retryMiddlewareDeclaration !== null) {
        middlewareDeclarations.push(retryMiddlewareDeclaration);
    }
    if (middlewareDeclarations.length === 0) {
        return null;
    }
    var reversedMiddlewareDeclarations = reverse(middlewareDeclarations);
    return {
        start: buildChainRings(first(reversedMiddlewareDeclarations), null, drop(reversedMiddlewareDeclarations)),
    };
};
export var executeChain = function (chain, fetchParams) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        return [2, chain.start.processRequest(fetchParams, null)];
    });
}); };
//# sourceMappingURL=chain.js.map