import * as tslib_1 from "tslib";
import { every, isNil, last, map, some } from "lodash";
var FetchyError = (function (_super) {
    tslib_1.__extends(FetchyError, _super);
    function FetchyError(message, responses, errors) {
        var _this = _super.call(this, message) || this;
        _this.failedResponses = null;
        _this.realErrors = null;
        Object.setPrototypeOf(_this, FetchyError.prototype);
        if (!isNil(responses)) {
            _this.saveResponses(responses);
        }
        if (!isNil(errors)) {
            _this.saveErrors(errors);
        }
        if (!_this.hasResponses() && !_this.hasErrors()) {
            throw new Error("Instances of FetchyError without response nor errors are not allowed.");
        }
        return _this;
    }
    Object.defineProperty(FetchyError.prototype, "responses", {
        get: function () {
            return this.failedResponses;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FetchyError.prototype, "errors", {
        get: function () {
            return this.realErrors;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FetchyError.prototype, "response", {
        get: function () {
            if (!this.hasResponses()) {
                throw new Error("No response to return. You should call this method after hasResponse");
            }
            return last(this.responses);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FetchyError.prototype, "error", {
        get: function () {
            if (!this.hasErrors()) {
                throw new Error("No error to return. You should call this method after hasErrors");
            }
            return last(this.errors);
        },
        enumerable: true,
        configurable: true
    });
    FetchyError.prototype.hasResponses = function () {
        return this.failedResponses !== null
            && this.failedResponses.length > 0;
    };
    FetchyError.prototype.hasErrors = function () {
        return this.realErrors !== null
            && this.realErrors.length > 0;
    };
    FetchyError.prototype.saveResponses = function (responses) {
        if (some(map(responses, "ok"))) {
            throw new Error("One or more responses is not failed.");
        }
        this.failedResponses = map(responses, function (response) { return response.clone(); });
    };
    FetchyError.prototype.saveErrors = function (errors) {
        if (!every(map(errors, function (error) { return error instanceof Error; }))) {
            throw new Error("One or more errors is not instance of Error.");
        }
        var normalizedErrors = map(errors, function (error) {
            if (error.name === "FetchError") {
                Object.setPrototypeOf(error, TypeError.prototype);
                error.name = "TypeError";
            }
            return error;
        });
        this.realErrors = normalizedErrors;
    };
    return FetchyError;
}(Error));
export { FetchyError };
//# sourceMappingURL=error.js.map