"use strict";
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var lodash_1 = require("lodash");
var fetch_1 = require("./middlewares/fetch");
var retry_1 = require("./middlewares/retry");
exports.validateMiddlewareDeclarations = function (middlewares) {
    return lodash_1.every(lodash_1.map(middlewares, function (middlewareDeclaration) {
        return lodash_1.has(middlewareDeclaration, "config")
            && !lodash_1.isNil(middlewareDeclaration.class);
    }));
};
var validateFetchyConfig = function (fetchyConfig) {
    if (lodash_1.isNil(fetchyConfig.middlewares)) {
        return [];
    }
    return exports.validateMiddlewareDeclarations(fetchyConfig.middlewares)
        ? fetchyConfig.middlewares
        : null;
};
var instanceFetchyMiddleware = function (middlewareClass, middlewareConfig, nextRing) {
    if (nextRing === null) {
        nextRing = new fetch_1.FetchFakeMiddleware({}, null);
    }
    return new middlewareClass(middlewareConfig, nextRing);
};
var buildChainRings = function (middlewareDeclaration, nextRing, remainingMiddlewareDeclarations) {
    var ring = instanceFetchyMiddleware(middlewareDeclaration.class, middlewareDeclaration.config, nextRing);
    if (remainingMiddlewareDeclarations.length > 0) {
        return buildChainRings(lodash_1.first(remainingMiddlewareDeclarations), ring, lodash_1.drop(remainingMiddlewareDeclarations, 1));
    }
    return ring;
};
exports.buildChain = function (fetchyConfig) {
    var middlewareDeclarations = validateFetchyConfig(fetchyConfig);
    if (middlewareDeclarations === null) {
        throw new Error("middlewares object is not valid");
    }
    var retryMiddlewareDeclaration = retry_1.getRetryMiddlewareDeclaration(fetchyConfig.retry);
    if (retryMiddlewareDeclaration !== null) {
        middlewareDeclarations.push(retryMiddlewareDeclaration);
    }
    if (middlewareDeclarations.length === 0) {
        return null;
    }
    var reversedMiddlewareDeclarations = lodash_1.reverse(middlewareDeclarations);
    return {
        start: buildChainRings(lodash_1.first(reversedMiddlewareDeclarations), null, lodash_1.drop(reversedMiddlewareDeclarations)),
    };
};
exports.executeChain = function (chain, fetchParams) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        return [2, chain.start.processRequest(fetchParams, null)];
    });
}); };
//# sourceMappingURL=chain.js.map