"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var lodash_1 = require("lodash");
var error_1 = require("../utils/error");
var base_1 = require("./base");
var DEFAULT_RETRY_CONFIG = {
    attempts: 5,
    backoff: 2,
    retriableStatusCodes: function (statusCode) { return statusCode >= 500; },
    retryNetworkErrors: false,
};
var FetchyRetryMiddleware = (function (_super) {
    tslib_1.__extends(FetchyRetryMiddleware, _super);
    function FetchyRetryMiddleware(config, next) {
        var _this = _super.call(this, config, next) || this;
        _this.attemptsCount = 0;
        _this.failedResponses = [];
        _this.errors = [];
        return _this;
    }
    FetchyRetryMiddleware.prototype.processRequest = function (fetchParams, previousMiddleware) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                this.fetchParamsClone = lodash_1.cloneDeep(fetchParams);
                return [2, _super.prototype.processRequest.call(this, fetchParams, previousMiddleware)];
            });
        });
    };
    FetchyRetryMiddleware.prototype.processResponse = function (promise0) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2, promise0
                        .then(function (response) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        return tslib_1.__generator(this, function (_a) {
                            if (!response.ok) {
                                throw new Error("The request is failed. This shouldn't happen.");
                            }
                            return [2, this.processNextResponse(new Promise(function (resolve) {
                                    resolve(response);
                                }))];
                        });
                    }); })
                        .catch(function (e) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var canRetry;
                        var _this = this;
                        return tslib_1.__generator(this, function (_a) {
                            canRetry = e instanceof error_1.FetchyError
                                && (e.hasErrors()
                                    || (e.hasResponses() && this.isRetriableCode(e.response.status)));
                            if (canRetry) {
                                if (e.hasErrors()) {
                                    this.errors = lodash_1.merge(this.errors, e.errors);
                                }
                                if (e.hasResponses()) {
                                    this.failedResponses = lodash_1.merge(this.failedResponses, e.responses);
                                }
                                this.attemptsCount++;
                                if (this.attemptsCount >= this.config.attempts) {
                                    throw new error_1.FetchyError("Too many failures (" + this.fetchParamsClone.input + ")", this.failedResponses, this.errors);
                                }
                                return [2, new Promise(function (resolve) {
                                        var seconds = Math.pow(_this.config.backoff, _this.attemptsCount) * 1000;
                                        setTimeout(function () {
                                            resolve(_this.processRequest(_this.fetchParamsClone, _this.previous));
                                        }, seconds);
                                    })];
                            }
                            throw e;
                        });
                    }); })];
            });
        });
    };
    FetchyRetryMiddleware.prototype.isRetriableCode = function (statusCode) {
        if (lodash_1.isArray(this.config.retriableStatusCodes)) {
            return !lodash_1.isUndefined(lodash_1.find(this.config.retriableStatusCodes, statusCode));
        }
        if (lodash_1.isFunction(this.config.retriableStatusCodes)) {
            return this.config.retriableStatusCodes(statusCode);
        }
        throw new TypeError("Retry config property \"retriableStatusCodes\" is not an array or a function");
    };
    return FetchyRetryMiddleware;
}(base_1.FetchyMiddleware));
exports.FetchyRetryMiddleware = FetchyRetryMiddleware;
exports.getRetryMiddlewareDeclaration = function (retry) {
    if (retry === false) {
        return null;
    }
    var config = retry === true ? DEFAULT_RETRY_CONFIG : retry;
    return {
        class: FetchyRetryMiddleware,
        config: config,
    };
};
//# sourceMappingURL=retry.js.map