"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var lodash_1 = require("lodash");
var FetchyError = (function (_super) {
    tslib_1.__extends(FetchyError, _super);
    function FetchyError(message, responses, errors) {
        var _this = _super.call(this, message) || this;
        _this.failedResponses = null;
        _this.realErrors = null;
        Object.setPrototypeOf(_this, FetchyError.prototype);
        if (!lodash_1.isNil(responses)) {
            _this.saveResponses(responses);
        }
        if (!lodash_1.isNil(errors)) {
            _this.saveErrors(errors);
        }
        if (!_this.hasResponses() && !_this.hasErrors()) {
            throw new Error("Instances of FetchyError without response nor errors are not allowed.");
        }
        return _this;
    }
    Object.defineProperty(FetchyError.prototype, "responses", {
        get: function () {
            return this.failedResponses;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FetchyError.prototype, "errors", {
        get: function () {
            return this.realErrors;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FetchyError.prototype, "response", {
        get: function () {
            if (!this.hasResponses()) {
                throw new Error("No response to return. You should call this method after hasResponse");
            }
            return lodash_1.last(this.responses);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FetchyError.prototype, "error", {
        get: function () {
            if (!this.hasErrors()) {
                throw new Error("No error to return. You should call this method after hasErrors");
            }
            return lodash_1.last(this.errors);
        },
        enumerable: true,
        configurable: true
    });
    FetchyError.prototype.hasResponses = function () {
        return this.failedResponses !== null
            && this.failedResponses.length > 0;
    };
    FetchyError.prototype.hasErrors = function () {
        return this.realErrors !== null
            && this.realErrors.length > 0;
    };
    FetchyError.prototype.saveResponses = function (responses) {
        if (lodash_1.some(lodash_1.map(responses, "ok"))) {
            throw new Error("One or more responses is not failed.");
        }
        this.failedResponses = lodash_1.map(responses, function (response) { return response.clone(); });
    };
    FetchyError.prototype.saveErrors = function (errors) {
        if (!lodash_1.every(lodash_1.map(errors, function (error) { return error instanceof Error; }))) {
            throw new Error("One or more errors is not instance of Error.");
        }
        var normalizedErrors = lodash_1.map(errors, function (error) {
            if (error.name === "FetchError") {
                Object.setPrototypeOf(error, TypeError.prototype);
                error.name = "TypeError";
            }
            return error;
        });
        this.realErrors = normalizedErrors;
    };
    return FetchyError;
}(Error));
exports.FetchyError = FetchyError;
//# sourceMappingURL=error.js.map